﻿
namespace TDD.Mocking
{
    // Main version of Class Under Test; constructor injection 
    public class OrderReader
    {
        private readonly IFileReader _fileReader;

        public OrderReader(IFileReader fileReader)
        {
            this._fileReader = fileReader;
        }

        public Order ReadOrder()
        {
            _fileReader.FileName = "record.csv";
            string line = _fileReader.ReadLine();
            _fileReader.Close();
            string[] details = line.Split(new char[] { ',' });
            int id = int.Parse(details[0]);
            string cust = details[1];
            double price = double.Parse(details[2]);
            Order order = new Order(id, cust, price);
            return order;
        }
    }
    /*
     One problem with constructor injection: what if there are multiple external dependencies?
     Have to add more & more parameters to constructor, or more & more constructor overloads
     -> code becomes less readable/maintainable 
     Solutions: 1) parameter object refactoring: create a class that contains all the values 
     needed, and just pass that single instance to the constructor
     2) Inversion of Control containers (Castle Windsor, Spring.NET, Ninjet, StructureMap, etc.) 
    */
}
