﻿
namespace TDD.Mocking
{
    // Osherove, section 3.4 "Refactoring our design to be more testable"
    // 2. Property injection: receive an interface as a property get or set
    // Like OrderReader0, we have a no-args constructor which creates the concrete object
    // Like OrderReader, the type of the variable is the interface; 
    public class OrderReader1
    {
        private IFileReader _fileReader;
        public OrderReader1()
        {
            _fileReader = new FileReader();
        }
        public IFileReader FileReader
        {
            get { return _fileReader; }
            set { _fileReader = value; }
        }

        public Order ReadOrder()
        {
            _fileReader.FileName = "record.csv";
            string line = _fileReader.ReadLine();
            _fileReader.Close();
            string[] details = line.Split(new char[] { ',' });
            int id = int.Parse(details[0]);
            string cust = details[1];
            double price = double.Parse(details[2]);
            Order order = new Order(id, cust, price);
            return order;
        }
    }
    /*
     Constructor injection signals in the API that the parameters are not optional: have to be 
     supplied when creating the object.  
     Use property injection when you want to signal that a dependency is optional (or that if there's 
     a dependency, there's a default instance which doesn't create any problems for tests) 
    */
}
