﻿using NUnit.Framework;
using Rhino.Mocks;

namespace TDD.Mocking {
    [TestFixture]
    class OrderReader2RhinoTest {
        MockRepository repository;
        IFileReader mockFileReader;
        OrderReader2 orderReader;

        [SetUp]
        public void SetUp() {
            repository = new MockRepository();
            // Set the mock to be the factory's FileReader
            mockFileReader = repository.StrictMock<IFileReader>();
            FileReaderFactory.SetReader(mockFileReader);
            orderReader = new OrderReader2();
        }

        [Test]
        public void TestReadOrder0() {
            Expect.Call(mockFileReader.FileName = "record.csv");
            Expect.Call(mockFileReader.ReadLine()).Return("27412,FUBAR,876543.21");
            mockFileReader.Close();
            LastCall.Repeat.Once();
            repository.ReplayAll();

            Order order = orderReader.ReadOrder();
            repository.VerifyAll();

            Assert.That(order.OrderId, Is.EqualTo(27412));
            Assert.That(order.Customer, Is.EqualTo("FUBAR"));
            Assert.That(order.Price, Is.EqualTo(876543.21));
        }
    }
}
