﻿using NUnit.Framework;
using Moq;

namespace TDD.Mocking
{
    // OrderReader3Aux (testable version of OrderReader3) defined in OrderReader3RhinoTest
    [TestFixture]
    class OrderReader3MoqTest
    {
        private Mock<IFileReader> mockFileReader; 
        // Declare instance of auxiliary subclass which is testable
        OrderReader3Aux orderReader;

        [SetUp]
        public void SetUp()
        {
            mockFileReader = new Mock<IFileReader>();
            orderReader = new OrderReader3Aux();
            // has a public variable we set directly to the mock object: 
            orderReader.fileReader = mockFileReader.Object;
        }

        [Test]
        public void ReadOrder_CreatesOrder()
        {
            mockFileReader.SetupSet(fr => fr.FileName = "record.csv");
            mockFileReader.Setup(fr => fr.ReadLine()).Returns("27412,FUBAR,876543.21").AtMostOnce();
            mockFileReader.Setup(fr => fr.Close());

            Order order = orderReader.ReadOrder();

            mockFileReader.VerifyAll();
            Assert.That(order.OrderId, Is.EqualTo(27412));
            Assert.That(order.Customer, Is.EqualTo("FUBAR"));
            Assert.That(order.Price, Is.EqualTo(876543.21));
        }

    }
}
