﻿using NUnit.Framework;
using NMock2;
using Is = NUnit.Framework.Is;


namespace TDD.Mocking {
    [TestFixture]
    class OrderReaderNMock2Test {
        [Test]
        public void TestReadOrder() {
            var mockery = new Mockery();
            var mockFileReader = (IFileReader)mockery.NewMock(typeof(IFileReader));
            Expect.Once.On(mockFileReader).SetProperty("FileName").To("record.csv");
            Expect.Once.On(mockFileReader).Method("ReadLine").WithNoArguments().Will(Return.Value("272,ABC,543.21"));
            Expect.Once.On(mockFileReader).Method("Close").WithNoArguments();

            OrderReader orderReader = new OrderReader(mockFileReader);
            Order order = orderReader.ReadOrder();
            mockery.VerifyAllExpectationsHaveBeenMet();

            Assert.That(order.OrderId, Is.EqualTo(272));
            Assert.That(order.Customer, Is.EqualTo("ABC"));
            Assert.That(order.Price, Is.EqualTo(543.21));
        }
    }
}
