﻿using NUnit.Framework;
using NUnit.Mocks;

namespace TDD.Mocking {
    // Uses the lightweight mocking API built into NUnit 
    [TestFixture]
    class OrderReaderNUnitTest {
        // Need the DynamicMock instance to Expect & Verify
        private DynamicMock dynamicMock;
        // Need it cast to IFileReader for OrderReader
        private IFileReader mockFileReader;
        private OrderReader orderReader;

        [SetUp]
        public void SetUp() {
            dynamicMock = new DynamicMock(typeof(IFileReader));
            mockFileReader = (IFileReader)dynamicMock.MockInstance;
            orderReader = new OrderReader(mockFileReader);
        }

        [Test]
        public void TestReadOrder() {
            // No special method for expecting property setter, 
            // so use underlying method for which it is syntactic suger
            dynamicMock.Expect("set_FileName", "record.csv");
            dynamicMock.ExpectAndReturn("ReadLine", "272,ABC,543.21");
            dynamicMock.Expect("Close");

            OrderReader orderReader = new OrderReader(mockFileReader);
            Order order = orderReader.ReadOrder();
            dynamicMock.Verify();

            Assert.That(order.OrderId, Is.EqualTo(272));
            Assert.That(order.Customer, Is.EqualTo("ABC"));
            Assert.That(order.Price, Is.EqualTo(543.21));
        }
    }
}
