﻿Namespace TDD.Mocking
    ' Starting point before refactoring: hard-coded dependency on concrete class 
    ' Cf. Osherove, section 3.4 "Refactoring our design to be more testable"
    ' 1. Receive an interface at the constructor level and save it in a field for later use
    ' = OrderReader
    ' NB the two constructors can be combined in a single class
    ' No args (OrderReader0) - default constructor creates object
    ' 1 arg (OrderReader) - receives interface; constructor can be called by tests
    ' Can put the 1 arg in a conditional compliation region so not visible in release mode
    ' #if DEBUG ... #endif
    ' Alternatively annotate with the attribute [Conditional("DEBUG")]
    ' Or make constructor internal rather than public 
    Class OrderReader0
        Private _fileReader As FileReader
        Public Sub New()
            _fileReader = New FileReader()
        End Sub

        Public Function ReadOrder() As Order
            _fileReader.FileName = "record.csv"
            Dim line As String = _fileReader.ReadLine()
            _fileReader.Close()
            Dim details As String() = line.Split(New Char() {","c})
            Dim id As Integer = Integer.Parse(details(0))
            Dim cust As String = details(1)
            Dim price As Double = Double.Parse(details(2))
            Dim order As New Order(id, cust, price)
            Return order
        End Function

    End Class
End Namespace
