﻿Namespace TDD.Mocking
    ' Osherove, section 3.4 "Refactoring our design to be more testable"
    ' 2. Property injection: receive an interface as a property get or set
    ' Like OrderReader0, we have a no-args constructor which creates the concrete object
    ' Like OrderReader, the type of the variable is the interface; 
    Public Class OrderReader1
        Private _fileReader As IFileReader
        Public Sub New()
            _fileReader = New FileReader()
        End Sub
        Public Property FileReader() As IFileReader
            Get
                Return _fileReader
            End Get
            Set(value As IFileReader)
                _fileReader = value
            End Set
        End Property

        Public Function ReadOrder() As Order
            _fileReader.FileName = "record.csv"
            Dim line As String = _fileReader.ReadLine()
            _fileReader.Close()
            Dim details As String() = line.Split(New Char() {","c})
            Dim id As Integer = Integer.Parse(details(0))
            Dim cust As String = details(1)
            Dim price As Double = Double.Parse(details(2))
            Dim order As New Order(id, cust, price)
            Return order
        End Function
    End Class
    '
    '     Constructor injection signals in the API that the parameters are not optional: have to be 
    '     supplied when creating the object.  
    '     Use property injection when you want to signal that a dependency is optional (or that if there's 
    '     a dependency, there's a default instance which doesn't create any problems for tests) 
    '    

End Namespace
