﻿Imports NUnit.Framework
Imports Moq

Namespace TDD.Mocking
    <TestFixture()> _
    Class OrderReader2MoqTest
        Private orderReader As OrderReader2
        Private mockFileReader As Mock(Of IFileReader)

        <SetUp()> _
        Public Sub SetUp()
            mockFileReader = New Mock(Of IFileReader)()
            ' Set the mock to be the factory's FileReader
            FileReaderFactory.SetReader(mockFileReader.[Object])
            orderReader = New OrderReader2()
        End Sub

        <Test()> _
        Public Sub ReadOrder_CreatesOrder()
            mockFileReader.SetupSet(Function(fr) InlineAssignHelper(fr.FileName, "record.csv"))
            mockFileReader.Setup(Function(fr) fr.ReadLine()).Returns("27412,FUBAR,876543.21").AtMostOnce()
            mockFileReader.Setup(Sub(fr) fr.Close())

            Dim order As Order = orderReader.ReadOrder()

            mockFileReader.VerifyAll()
            Assert.That(order.OrderId, [Is].EqualTo(27412))
            Assert.That(order.Customer, [Is].EqualTo("FUBAR"))
            Assert.That(order.Price, [Is].EqualTo(876543.21))
        End Sub
        Private Shared Function InlineAssignHelper(Of T)(ByRef target As T, value As T) As T
            target = value
            Return value
        End Function

    End Class
End Namespace
