﻿Imports NUnit.Framework
Imports Moq

Namespace TDD.Mocking
    ' OrderReader3Aux (testable version of OrderReader3) defined in OrderReader3RhinoTest
    <TestFixture()> _
    Class OrderReader3MoqTest
        Private mockFileReader As Mock(Of IFileReader)
        ' Declare instance of auxiliary subclass which is testable
        Private orderReader As OrderReader3Aux

        <SetUp()> _
        Public Sub SetUp()
            mockFileReader = New Mock(Of IFileReader)()
            orderReader = New OrderReader3Aux()
            ' has a public variable we set directly to the mock object: 
            orderReader.fileReader = mockFileReader.[Object]
        End Sub

        <Test()> _
        Public Sub ReadOrder_CreatesOrder()
            mockFileReader.SetupSet(Function(fr) InlineAssignHelper(fr.FileName, "record.csv"))
            mockFileReader.Setup(Function(fr) fr.ReadLine()).Returns("27412,FUBAR,876543.21").AtMostOnce()
            mockFileReader.Setup(Sub(fr) fr.Close())

            Dim order As Order = orderReader.ReadOrder()

            mockFileReader.VerifyAll()
            Assert.That(order.OrderId, [Is].EqualTo(27412))
            Assert.That(order.Customer, [Is].EqualTo("FUBAR"))
            Assert.That(order.Price, [Is].EqualTo(876543.21))
        End Sub
        Private Shared Function InlineAssignHelper(Of T)(ByRef target As T, value As T) As T
            target = value
            Return value
        End Function

    End Class
End Namespace
