﻿Imports NUnit.Framework
Imports Rhino.Mocks

Namespace TDD.Mocking
    '
    '     "Extract and Override is great for simulating inputs into your code under test, but it's 
    '     cumbersome when you want to verify interactions that are coming out of the code under test
    '     into your dependency."  Osherove, p. 74.  
    '     


    <TestFixture()> _
    Class OrderReader3RhinoTest
        Private repository As MockRepository
        Private mockFileReader As IFileReader
        ' Declare instance of auxiliary subclass which is testable
        Private orderReader As OrderReader3Aux

        <SetUp()> _
        Public Sub SetUp()
            repository = New MockRepository()
            mockFileReader = repository.StrictMock(Of IFileReader)()
            orderReader = New OrderReader3Aux()
            ' has a public variable we set directly to the mock object: 
            orderReader.fileReader = mockFileReader
        End Sub

        <Test()> _
        Public Sub TestReadOrder0()
            Rhino.Mocks.Expect.Call(mockFileReader.FileName).SetPropertyWithArgument("record.csv")
            Rhino.Mocks.Expect.Call(mockFileReader.ReadLine()).Return("27412,FUBAR,876543.21")
            mockFileReader.Close()
            LastCall.Repeat.Once()
            repository.ReplayAll()

            Dim order As Order = orderReader.ReadOrder()
            repository.VerifyAll()

            Assert.That(order.OrderId, [Is].EqualTo(27412))
            Assert.That(order.Customer, [Is].EqualTo("FUBAR"))
            Assert.That(order.Price, [Is].EqualTo(876543.21))
        End Sub

    End Class
End Namespace
