﻿Imports NUnit.Framework
Imports Rhino.Mocks
Imports System.IO

Namespace TDD.Mocking
    <TestFixture()> _
    Class OrderReaderRhinoTest
        Private repository As MockRepository
        Private mockFileReader As IFileReader
        Private orderReader As OrderReader

        <SetUp()> _
        Public Sub SetUp()
            repository = New MockRepository()
            ' Obsolete method: 
            ' mockFileReader = repository.CreateMock<IFileReader>();
            mockFileReader = repository.StrictMock(Of IFileReader)()
            ' mockFileReader = repository.DynamicMock<IFileReader>();
            orderReader = New OrderReader(mockFileReader)
        End Sub

        ' 4 styles - "classic"
        <Test()> _
        Public Sub TestReadOrder0()
            Rhino.Mocks.Expect.Call(mockFileReader.FileName).SetPropertyWithArgument("record.csv")
            Rhino.Mocks.Expect.Call(mockFileReader.ReadLine()).Return("27412,FUBAR,876543.21")
            mockFileReader.Close()
            LastCall.Repeat.Once()
            repository.ReplayAll()

            Dim order As Order = orderReader.ReadOrder()
            repository.VerifyAll()

            Assert.That(order.OrderId, [Is].EqualTo(27412))
            Assert.That(order.Customer, [Is].EqualTo("FUBAR"))
            Assert.That(order.Price, [Is].EqualTo(876543.21))
        End Sub

        <Test()> _
        Public Sub TestReadOrder1()
            Using repository.Record()
                'using (repository.Ordered())
                ' method which returns a value: 
                Rhino.Mocks.Expect.Call(mockFileReader.ReadLine()).Return("27412,FUBAR,876543.21")
                ' property: 
                Rhino.Mocks.Expect.Call(mockFileReader.FileName).SetPropertyWithArgument("record.csv")
                ' void method: 
                Rhino.Mocks.Expect.Call(Sub() mockFileReader.Close())
            End Using

            'Order order;
            'using (repository.Playback())
            '{
            '    order = orderReader.ReadOrder();
            '}
            Dim order As Order = orderReader.ReadOrder()
            repository.VerifyAll()

            Assert.That(order.OrderId, [Is].EqualTo(27412))
            Assert.That(order.Customer, [Is].EqualTo("FUBAR"))
            Assert.That(order.Price, [Is].EqualTo(876543.21))
        End Sub

        <Test()> _
        Public Sub TestReadOrder2()
            Dim order As Order = Nothing
            ' Lambda syntax: 
            [With].Mocks(repository).Expecting(Sub()
                                                   Rhino.Mocks.Expect.Call(mockFileReader.FileName).SetPropertyWithArgument("record.csv")
                                                   Rhino.Mocks.Expect.Call(mockFileReader.ReadLine()).Return("27412,ABC,876543.21")
                                                   mockFileReader.Expect(Sub(x) x.Close())

                                               End Sub).Verify(Sub() order = orderReader.ReadOrder())
            Assert.That(order.OrderId, [Is].EqualTo(27412))
            Assert.That(order.Customer, [Is].EqualTo("ABC"))
            Assert.That(order.Price, [Is].EqualTo(876543.21))
        End Sub

        '''/ Arrange, Act, Assert
        '[Test]
        'public void TestReadOrder3()
        '{
        '    // var mockFR = MockRepository.GenerateMock<IFileReader>();
        '    // If we want Ordered in AAA syntax: 
        '    // mockFR.GetMockRepository().Ordered();
        '    // For a stub: 
        '    var mockFR = MockRepository.GenerateStub<IFileReader>();

        '    // for a stub, property getter like this: 
        '    mockFR.FileName = "record.csv";
        '    // mockFR.Expect(x => x.FileName).Return("record.csv");

        '    string rv = "27412,FUBAR,876543.21";
        '    mockFR.Expect(x => x.ReadLine()).Return(rv);
        '    mockFR.Stub(x => x.Close());

        '    // mockFR.BackToRecord();
        '    // mockFR.Replay();

        '    Order order = orderReader.ReadOrder();

        '    Assert.That(order.OrderId, Is.EqualTo(27412));
        '    Assert.That(order.Customer, Is.EqualTo("FUBAR"));
        '    Assert.That(order.Price, Is.EqualTo(876543.21));

        '    // mockFR.VerifyAllExpectations();
        '}

        <Test(), ExpectedException(GetType(IOException))> _
        Public Sub TestFileReaderThrowsUp()
            Using repository.Record()
                Rhino.Mocks.Expect.Call(mockFileReader.FileName).SetPropertyWithArgument("record.csv")
                ' mock the file reader implementation throwing an IOEx:
                Rhino.Mocks.Expect.Call(Sub() mockFileReader.ReadLine()).Throw(New IOException("Bad file"))
                ' we don't expect Close() to be called: 
                mockFileReader.Close()
                LastCall.Repeat.Never()
            End Using

            Dim order As Order
            Using repository.Playback()
                order = orderReader.ReadOrder()
            End Using
        End Sub

    End Class
End Namespace
